/** \mainpage	Project1 pagina
 *
 *	Project1 main page
 *
 *
 * \section PsoIntro1	Introduction. Part 1
 *
 *	In computer science, particle swarm optimization (PSO) is a computational
 * method that optimizes a problem by iteratively trying to improve a candidate
 * solution with regard to a given measure of quality. PSO optimizes a problem
 * by having a population of candidate solutions, here dubbed particles, and
 * moving these particles around in the search-space according to simple
 * mathematical formulae over the particle's position and velocity. Each
 * particle's movement is influenced by its local best known position and it's
 * also guided toward the best known positions in the search-space, which are
 * updated as better positions are found by other particles. This is expected
 * to move the swarm toward the best solutions.
 *
 * \section PsoIntro2	Introduction. Part 2
 *
 * PSO is originally attributed to Kennedy, Eberhart and Shi[1][2] and was first
 * intended for simulating social behaviour,[3] as a stylized representation of
 * the movement of organisms in a bird flock or fish school. The algorithm was
 * simplified and it was observed to be performing optimization. The book by
 * Kennedy and Eberhart[4] describes many philosophical aspects of PSO and swarm
 * intelligence. An extensive survey of PSO applications is made by Poli.[5][6]
 *
 * \section PsoIntro3	Introduction. Part 3
 *
 * PSO is a metaheuristics as it make few or no assumptions about the problem
 * being optimized and can search very large spaces of candidate solutions.
 * However, metaheuristics such as PSO do not guarantee an optimal solution is
 * ever found. More specifically, PSO does not use the gradient of the problem
 * being optimized, which means PSO does not require that the optimization
 * problem be differentiable as is required by classic optimization methods such
 * as gradient descent and quasi-newton methods. PSO can therefore also be used
 * on optimization problems that are partially irregular, noisy, change over
 * time, etc.
 *
 * \section PsoAlg		Algorithm
 *
 * A basic variant of the PSO algorithm works by having a population (called a
 * swarm) of candidate solutions (called particles). These particles are moved
 * around in the search-space according to a few simple formulae. The movements
 * of the particles are guided by their own best known position in search-space
 * as well as the entire swarm's best known position. When improved positions
 * are being discovered these will then come to guide the movements of the swarm.
 * The process is repeated and by doing so it is hoped, but not guaranteed, that
 * a satisfactory solution will eventually be discovered.
 *
 * Formally, let f: R<sup>n</sup> -> R be the fitness or cost function which must
 * be minimized. The function takes a candidate solution as argument in the form
 * of a vector of real numbers and produces a real number as output which
 * indicates the fitness of the given candidate solution. The gradient of f is
 * not known. The goal is to find a solution a for which f(a) <= f(b) for all b
 * in the search-space, which would mean a is the global minimum. Maximization
 * can be performed by considering the function h = -f instead.
 *
 * Let S be the number of particles in the swarm, each having a position
 * x<sub>i</sub> in the search-space and a velocity v<sub>i</sub> (both of them
 * belonging to R<sup>n</sup>). Let p<sub>i</sub> be the best known position of
 * particle i and let g be the best known position of the entire swarm. A basic
 * PSO algorithm is then:
 *
 * <ul>
 * <li> For each particle i = 1, ..., S do:
 *    <ul>
 *    <li> Initialize the particle's position with a uniformly distributed random vector: b<sub>lo</sub> <= x<sub>i</sub> <= b<sub>up</sub>, where b<sub>lo</sub> and b<sub>up</sub> are the lower and upper boundaries of the search-space.
 *    <li> Initialize the particle's best known position to its initial position: p<sub>i</sub> = x<sub>i</sub>
 *    <li> If (f(p<sub>i</sub>) < f(g)) update the swarm's best known position: g = p<sub>i</sub>
 *    <li> Initialize the particle's velocity: -|b<sub>up</sub>-b<sub>lo</sub>| <= v<sub>i</sub> <= |b<sub>up</sub>-b<sub>lo</sub>|
 *    </ul>
 * <li> Until a termination criterion is met (e.g. number of iterations
 *		performed, or adequate fitness reached), repeat:
 *	  <ul>
 *	  <li> For each particle i = 1, ..., S do:
 *       <ul>
 *       <li> Pick random numbers: 0 <= rp, rg <= 1
 *       <li> Update the particle's velocity: v<sub>i</sub> = I * v<sub>i</sub> + C<sub>p</sub> * r<sub>p</sub> * (p<sub>i</sub>-x<sub>i</sub>) + C<sub>g</sub> * r<sub>g</sub> * (g-x<sub>i</sub>)
 *       <li> Update the particle's position: x<sub>i</sub> = x<sub>i</sub> + v<sub>i</sub>
 *       <li> If (f(x<sub>i</sub>) < f(p<sub>i</sub>)) do:
 *          <ul>
 *          <li> Update the particle's best known position: p<sub>i</sub> = x<sub>i</sub>
 *          <li> If (f(p<sub>i</sub>) < f(g)) update the swarm's best known position: g = p<sub>i</sub>
 *          </ul>
 *       </ul>
 *    </ul>
 * <li> Now g holds the best found solution.
 * </ul>
 *
 * The parameters I, C<sub>p</sub>, and C<sub>g</sub> are selected by the
 * practitioner and control the behaviour and efficacy of the PSO method, see
 * below.
 */
